"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const write_file_atomic_1 = __importDefault(require("write-file-atomic"));
const lodash_has_1 = __importDefault(require("lodash.has"));
const lodash_get_1 = __importDefault(require("lodash.get"));
const lodash_set_1 = __importDefault(require("lodash.set"));
const lodash_unset_1 = __importDefault(require("lodash.unset"));
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
// -----------------------------------------------------------------------------
// TeenyConf itself
// -----------------------------------------------------------------------------
class TeenyConf {
    _configPath;
    _defaultConfig;
    _conf;
    constructor(configPath, defaultConfig) {
        if (!configPath)
            throw new TypeError("teenyconf needs a valid configPath");
        this._configPath = path.resolve(configPath);
        this._defaultConfig = defaultConfig;
        // Check if directory exists, creates it if needed
        if (!fs.existsSync(path.dirname(this._configPath))) {
            fs.mkdirSync(path.dirname(this._configPath));
        }
        try {
            // Load the file
            this._conf = JSON.parse(fs.readFileSync(this._configPath).toString());
        }
        catch (err) {
            // console.info(`An error occured when parsing ${_configPath}, fallback on default config`);
            this._conf = (0, lodash_clonedeep_1.default)(this._defaultConfig);
            this.save();
        }
    }
    /**
     * Reload the configuration from file
     */
    reload() {
        this._conf = JSON.parse(fs.readFileSync(this._configPath).toString());
    }
    /**
     * Save current config to its associated file
     */
    save(minify = false) {
        const output = minify
            ? JSON.stringify(this._conf)
            : JSON.stringify(this._conf, null, " ");
        write_file_atomic_1.default.sync(this._configPath, output);
    }
    get(key, def) {
        if (key) {
            return (0, lodash_get_1.default)(this._conf, key, def);
        }
        // Else return everything
        return this._conf;
    }
    getx(key, def) {
        if (key) {
            const valueExists = (0, lodash_has_1.default)(this._conf, key);
            if (valueExists) {
                // we checked with has() first, so it's safe
                return (0, lodash_get_1.default)(this._conf, key, def);
            }
        }
        throw new Error(`TeenyConf - key "${key}" does not exist in the config.`);
    }
    /**
     * Set/add key/value pair
     */
    set(key, value) {
        (0, lodash_set_1.default)(this._conf, key, value);
    }
    /**
     * Delete a key from the configuration
     */
    delete(key) {
        (0, lodash_unset_1.default)(this._conf, key);
    }
    /**
     * Clear the configuration and rolls it back the default config
     */
    clear() {
        this._conf = (0, lodash_clonedeep_1.default)(this._defaultConfig);
    }
    /**
     * Check if a key exists
     */
    has(key) {
        return (0, lodash_has_1.default)(this._conf, key);
    }
}
exports.default = TeenyConf;
//# sourceMappingURL=teeny-conf.js.map